﻿using System;
using System.Diagnostics;
using System.Xml;
using ESVBible.Service;
using ESVBible.Service.Parameter;
using ESVBible.Service.Setting;
using Microsoft.VisualStudio.TestTools.UnitTesting;
//+
namespace ESVBible.Demo
{
    [TestClass]
    /// <summary>
    /// These are demos, not unit tests
    /// 
    /// For more information, see:
    ///   http://www.netfxharmonics.com/2007/11/ESV-Bible-Web-Service-20-Framework-for-NET-35
    ///   
    /// </summary>
    public class ESVBible : TestBase
    {
        //- @SimplePassageQueryTest -//
        [TestMethod]
        public void SimplePassageQueryTest()
        {
            ESVBibleServiceV2 service = new ESVBibleServiceV2();
            String output = service.PassageQuery("Galatians 3:11");
            //+
            Assert.IsFalse(String.IsNullOrEmpty(output));
        }

        //- @SimplePassageQueryPlainTextTest -//
        [TestMethod]
        public void SimplePassageQueryPlainTextTest()
        {
            ESVBibleServiceV2 service = new ESVBibleServiceV2(OutputFormat.PlainText);
            String output = service.PassageQuery("Galatians 3:11");
            //+
            Assert.IsFalse(String.IsNullOrEmpty(output));
        }

        //- @SimplePassageQueryWithParametersTest -//
        [TestMethod]
        public void SimplePassageQueryWithParametersTest()
        {
            PassageQueryParameter pqp = new PassageQueryParameter() { Passage = "John 14:6" };
            ESVBibleServiceV2 service = new ESVBibleServiceV2(new PlainTextSetting()
            {
                LineLength = 100,
                Timeout = 30
            });
            String output = service.PassageQuery(pqp);
            //+
            Assert.IsFalse(String.IsNullOrEmpty(output));
        }

        //- @DailyVerseTest -//
        [TestMethod]
        public void DailyVerseTest()
        {
            ESVBibleServiceV2 service = new ESVBibleServiceV2(new HtmlOutputSetting()
            {
                IncludeAudioLink = false
            });
            String output = service.DailyVerse();
            //+
            Assert.IsFalse(String.IsNullOrEmpty(output));
        }

        //- @ReadingPlanTest -//
        [TestMethod]
        public void ReadingPlanTest()
        {
            ESVBibleServiceV2 service = new ESVBibleServiceV2();
            String output = service.ReadingPlanQuery(new ReadingPlanQueryParameter()
            {
                ReadingPlan = ReadingPlan.EveryDayInTheWord
            });
            //+
            Assert.IsFalse(String.IsNullOrEmpty(output));
        }

        //- @SearchTest -//
        [TestMethod]
        public void SearchTest()
        {
            ESVBibleServiceV2 service = new ESVBibleServiceV2();
            String output = service.Query("Justified");
            //+
            Assert.IsFalse(String.IsNullOrEmpty(output));
        }

        //- @QueryInfoAsObjectTest -//
        [TestMethod]
        public void QueryInfoAsObjectTest()
        {
            ESVBibleServiceV2 service = new ESVBibleServiceV2();
            QueryInfoData result = service.QueryInfoAsObject("Samuel");
            Assert.AreEqual<String>("1 Samuel 1", result.Readable);
            Assert.AreEqual<Int32>(131, result.AlternateResultCount);
            //+
            Trace.WriteLine(result.QueryType);
            Trace.WriteLine("----------------------");
            if (result.QueryType == QueryType.Passage)
            {
                Trace.WriteLine("Passage: " + result.Readable);
                Trace.WriteLine("Complete Chapter?: " + result.IsCompleteChapter);
                if (result.AlternateQueryType != QueryType.None)
                {
                    Trace.WriteLine(String.Format("Alternate: {0}, {1}", result.AlternateQueryType, result.AlternateResultCount));
                }
            }
            if (result.HasWarnings)
            {
                foreach (Warning w in result.Warnings)
                {
                    Trace.WriteLine(String.Format("{0}: {1}", w.Code, w.Readable));
                }
            }
        }

        //- @CrossWayXmlTest -//
        [TestMethod]
        public void CrossWayXmlTest()
        {
            ESVBibleServiceV2 service = new ESVBibleServiceV2(new CrosswayXmlVersion10Setting()
            {
                IncludeWordIds = true,
                IncludeXmlDeclaration = true
            });
            String output = service.PassageQuery(new PassageQueryParameter()
            {
                Passage = "Galatians 3"
            });
            //+
            Trace.WriteLine(output);
            //+
            Assert.IsFalse(String.IsNullOrEmpty(output));
        }

        //- @PassageQueryAsXmlDocumentTest -//
        [TestMethod]
        public void PassageQueryAsXmlDocumentTest()
        {
            ESVBibleServiceV2 service = new ESVBibleServiceV2();
            XmlDocument output = service.PassageQueryAsXmlDocument("Galatians 3");
            //+
            Assert.IsFalse(String.IsNullOrEmpty(output.InnerXml));
        }

        //- @PassageQueryValueViaXPathTest -//
        [TestMethod]
        public void PassageQueryValueViaXPathTest()
        {
            ESVBibleServiceV2 service = new ESVBibleServiceV2();
            String output = service.PassageQueryValueViaXPath(new PassageQueryParameter()
            {
                Passage = "Gal 3:4-5",
                XPath = "//crossway-bible/passage/surrounding-chapters/current"
            });
            //+
            Assert.IsFalse(String.IsNullOrEmpty(output));
        }

        //- @PassageQueryValueViaXPathMultiTest -//
        [TestMethod]
        public void PassageQueryValueViaXPathMultiTest()
        {
            ESVBibleServiceV2 service = new ESVBibleServiceV2();
            String[] output = service.PassageQueryValueViaXPathMulti(new PassageQueryParameter()
            {
                Passage = "Gal 3:4-5",
                XPathSet = new[]
                {
                    "//crossway-bible/passage/surrounding-chapters/previous",
                    "//crossway-bible/passage/surrounding-chapters/next"                
                }
            });
            //+
            Assert.IsTrue(output.Length == 2);
        }
    }
}