using System;
using System.Collections.Generic;
//+
namespace ESVBible.Service.Parameter
{
    /// <summary>
    /// Parameters representing the dailyVerse function, providing a new verse daily.
    /// </summary>
    public class DailyVerseParameter : Parameter, IDefaultOptimization
    {

        //- @BeginCharacter -//
        /// <summary>
        /// If the first letter of the verse is lower-case, indicating that the verse begins in the middle of a sentence, you can prepend a string to the verse. For example, you might set this option to &ellipsis; or ... to begin such verses with an ellipsis. (default: An empty string)
        /// </summary>
        public String BeginCharacter { get; set; }

        //- @CorrectCapitalization -//
        /// <summary>
        /// If the first letter of the verse is lower-case, setting this option to true will upper-case it. (default: true)
        /// </summary>
        public Boolean CorrectCapitalization { get; set; }

        //- @CorrectEndPunctuation -//
        /// <summary>
        /// If the verse ends with anything besides a period, question mark, or exclamation mark (for example, a comma, semicolon, or dash), it replaces the ending punctuation with whatever you set in end-character. (default: true)
        /// </summary>
        public Boolean CorrectEndPunctuation { get; set; }

        //- @CorrectQuotes -//
        /// <summary>
        /// Balances quotes. If a quotation beginning in a previous verse ends in the current verse, an opening quotation mark is added to the beginning of the verse. Similarly, an ending quotation mark is added to the end of the verse if the quotation begins in the verse but extends beyond the end of the verse. So, all quotations have both beginning and ending marks. (default: true)
        /// </summary>
        public Boolean CorrectQuotes { get; set; }

        //- @EndCharacter -//
        /// <summary>
        /// If the verse ends with anything besides a period, question mark, or exclamation mark (for example, a comma, semicolon, or dash), the ending punctuation will be replaced with this string. (default: &ellipsis4;)
        /// </summary>
        public String EndCharacter { get; set; }

        //- @IncludeHeadings -//
        /// <summary>
        /// Whether to include the section headings (e.g., Jesus Feeds the Five Thousand in John 6). (default: false)
        /// </summary>
        public Boolean IncludeHeadings { get; set; }

        //+
        //- $IncludeHeadingsDefault -//
        private Boolean IncludeHeadingsDefault { get; set; }

        //- $BeginCharacterDefault -//
        private String BeginCharacterDefault { get; set; }

        //- $CorrectCapitalizationDefault -//
        private Boolean CorrectCapitalizationDefault { get; set; }

        //- $CorrectEndPunctuationDefault -//
        private Boolean CorrectEndPunctuationDefault { get; set; }

        //- $CorrectQuotesDefault -//
        private Boolean CorrectQuotesDefault { get; set; }

        //- $EndCharacterDefault -//
        private String EndCharacterDefault { get; set; }

        //+
        //- @Ctor -//
        public DailyVerseParameter()
        {
            this.IncludeHeadings = false;
            this.CorrectCapitalization = true;
            this.CorrectEndPunctuation = true;
            this.CorrectQuotes = true;

            this.IncludeHeadingsDefault = false;
            this.CorrectCapitalizationDefault = true;
            this.CorrectEndPunctuationDefault = true;
            this.CorrectQuotesDefault = true;
        }

        //+
        //- $IDefaultOptimization.GetSettingsList -//
        Dictionary<String, String> IDefaultOptimization.GetSettingsList()
        {
            Dictionary<String, String> d = new Dictionary<String, String>();
            //+
            if (this.IncludeHeadings != this.IncludeHeadingsDefault) ArgumentListManager.AddArgument(d, "include-headings", this.IncludeHeadings);
            if (!String.IsNullOrEmpty(this.BeginCharacter)) ArgumentListManager.AddArgument(d, "begin-character", this.BeginCharacter);
            if (this.CorrectCapitalization != this.CorrectCapitalization) ArgumentListManager.AddArgument(d, "correct-capitalization", this.CorrectCapitalization);
            if (this.CorrectEndPunctuation != this.CorrectEndPunctuation) ArgumentListManager.AddArgument(d, "correct-end-punctuation", this.CorrectEndPunctuation);
            if (!String.IsNullOrEmpty(this.EndCharacter)) ArgumentListManager.AddArgument(d, "end-character", this.EndCharacter);
            if (this.CorrectQuotes != this.CorrectQuotes) ArgumentListManager.AddArgument(d, "correct-quotes", this.CorrectQuotes);
            //+
            return d;
        }
    }
}