using System;
using System.Collections.Generic;
//+
namespace ESVBible.Service.Parameter
{
    /// <summary>
    /// Parameters representing the query function, looking up a passage or showing word-search results, depending on the query.
    /// </summary>
    public class QueryParameter : Parameter, IDefaultOptimization
    {
        private String q;

        //+
        //- @Q -//
        /// <summary>
        /// Keywords with which to query.
        /// </summary>
        public String Q
        {
            get
            {
                if (String.IsNullOrEmpty(q))
                {
                    throw new ESVBibleException("Q (the string to search for) is required");
                }
                return q;
            }
            set
            {
                q = value;
            }
        }

        //+
        //- @LinkUrl -//
        /// <summary>
        /// Specifies what link to use when linking from the search results page to another page. If the page that handles your requests is named bible.search.php, for example, set this option to bible.search.php. (default: /esv/search/)
        /// </summary>
        public String LinkUrl { get; set; }

        //- @NotWords -//
        /// <summary>
        /// Exclude verses containing certain words. Can only be used in conjunction with the words and phrase options.
        /// </summary>
        public String NotWords { get; set; }

        //- @Matches -//
        /// <summary>
        /// [Empty] matches anywhere in words (the finds father). beginning matches beginnings of words (the finds there). exact matches exact words only (the only finds the). The latter two options are slower.
        /// </summary>
        public String Matches { get; set; }

        //- @Scope -//
        /// <summary>
        /// Limit your search to certain sections of the Bible. Accepts the predefined inputs listed in the dropdown menu on the Advanced Search page at http://www.gnpcb.org/esv/advanced.search/. (default: [Complete Bible])
        /// </summary>
        public String Scope { get; set; }

        //- @SearchText -//
        /// <summary>
        /// 'all' searches the complete text of the Bible, including headings and footnotes. Set to text to exclude headings and footnotes.( default: all)
        /// </summary>
        public String SearchText { get; set; }

        //- @Page -//
        /// <summary>
        /// If the results span more than one page (e.g., 27 results were found, but only 25 are shown per page), set the page number (starting at 1) to show a different page of results. (default: 1)
        /// </summary>
        public Int32 Page { get; set; }

        //- @Passage -//
        /// <summary>
        /// Put a string in here to force a passage search. This behavior is otherwise identical to passageQuery.
        /// </summary>
        public String Passage { get; set; }

        //- @Phrase -//
        /// <summary>
        /// Search for an exact phrase. In general, you can also surround words in q with quotation marks to mimic this functionality.
        /// </summary>
        public String Phrase { get; set; }

        //- @ResultsPerPage -//
        /// <summary>
        /// How many search results to show per page. Must be between 2 and 100 (inclusive). (default: 24)
        /// </summary>
        public Int32 ResultsPerPage { get; set; }

        //- @Words -//
        /// <summary>
        /// Put a string in here to force a word search.
        /// </summary>
        public String Words { get; set; }

        //+
        //- $PageDefault -//
        private Int32 PageDefault { get; set; }

        //- $ResultsPerPageDefault -//
        private Int32 ResultsPerPageDefault { get; set; }

        //+
        //- @QueryParameters -//
        public QueryParameter()
        {
            this.Page = 1;
            this.ResultsPerPage = 25;

            this.PageDefault = 1;
            this.ResultsPerPageDefault = 25;
        }

        //+
        //- $IDefaultOptimization.GetSettingsList -//
        Dictionary<String, String> IDefaultOptimization.GetSettingsList()
        {
            Dictionary<String, String> d = new Dictionary<String, String>();
            //+
            if (!String.IsNullOrEmpty(this.Q)) ArgumentListManager.AddArgument(d, "q", this.Q);
            if (!String.IsNullOrEmpty(this.Passage)) ArgumentListManager.AddArgument(d, "passage", this.Passage);
            if (!String.IsNullOrEmpty(this.Words)) ArgumentListManager.AddArgument(d, "words", this.Words);
            if (!String.IsNullOrEmpty(this.Phrase)) ArgumentListManager.AddArgument(d, "phrase", this.Phrase);
            if (!String.IsNullOrEmpty(this.NotWords)) ArgumentListManager.AddArgument(d, "not-words", this.NotWords);
            if (!String.IsNullOrEmpty(this.Scope)) ArgumentListManager.AddArgument(d, "scope", this.Scope);
            if (!String.IsNullOrEmpty(this.Matches)) ArgumentListManager.AddArgument(d, "matches", this.Matches);
            if (!String.IsNullOrEmpty(this.SearchText)) ArgumentListManager.AddArgument(d, "search-text", this.SearchText);
            if (this.Page != this.PageDefault) ArgumentListManager.AddArgument(d, "page", this.Page);
            if (!String.IsNullOrEmpty(this.LinkUrl)) ArgumentListManager.AddArgument(d, "link-url", this.LinkUrl);
            if (this.ResultsPerPage != this.ResultsPerPageDefault) ArgumentListManager.AddArgument(d, "results-per-page", this.ResultsPerPage);
            //+
            return d;
        }
    }
}