using System;
using System.Collections.Generic;
//+
namespace ESVBible.Service.Setting
{
    public class CrosswayXmlVersion10Setting : IOutputSetting, IDefaultOptimization
    {
        //- @BaseElement -//
        /// <summary>
        /// Set to paragraph to transform the output from verse-unit-based markup to paragraph-based markup. In doing so, you lose a little bit of information--you can no longer tell whether the end of a passage indicates the true end of a paragraph or block indent in the text, or whether it was put there to allow the proper closing of tags. That is to say, there is no longer 'end-(paragraph|block-indent) class="virtual"/'. (default: verse-unit)
        /// </summary>
        public String BaseElement { get; set; }

        //- @IncludeCrossReferences -//
        /// <summary>
        /// Currently, the cross-references don't point to anything, so there's little reason to include them. (default: false)
        /// </summary>
        public Boolean IncludeCrossReferences { get; set; }

        //- @IncludeDocType -//
        /// <summary>
        /// Include a DOCTYPE declaration. Note that it links to a DTD that only explains the character entities, not the elements. (default: true)
        /// </summary>
        public Boolean IncludeDocType { get; set; }

        //- @IncludeLineBreaks -//
        /// <summary>
        /// Include line breaks in the XML output. It doesn't make any functional difference; it's just easier to read. (default: false)
        /// </summary>
        public Boolean IncludeLineBreaks { get; set; }

        //- @IncludeQuoteEntities -//
        /// <summary>
        /// The original XML files contain an elaborate system of marking where quotations begin and end. This information is unnecessary for most purposes. Leaving this option set to true transforms the quotations from 'q' elements to the entities referenced in the DTD (e.g., 'ldblquot' for an opening double quotation mark). Setting this option to false leaves in the 'q' tags, as described in the XML Schema. (default: true)
        /// </summary>
        public Boolean IncludeQuoteEntities { get; set; }

        //- @IncludeSimpleEntities -//
        /// <summary>
        /// Translates character entities to native XML entities (e.g., 'ldblquot' becomes 'quot') for non-DTD-aware parsers. You can see the specific transformations we do in our PHP function translate_entities_to_simple_xml(). If you also disable the option include-quote-entities, the quote entities are translated to 'q' tags before applying this transformation. (default: false)
        /// </summary>
        public Boolean IncludeSimpleEntities { get; set; }

        //- @IncludeWordIds -//
        /// <summary>
        /// Include word ids ('w' tags) around each word in the text. The wid attribute consists of an eight-digit verse identifier, followed by a period (.) and a two-digit word identifier. The wids are unique for each word in the Bible; however, if you call the same passage more than once, the wids will repeat. In other words, two instances of Matthew 1:2 will have 'w' tags with identical wid values. (default: false)
        /// </summary>
        public Boolean IncludeWordIds { get; set; }

        //- @IncludeVirtualAttributes -//
        /// <summary>
        /// All begin- and end- tags must match--for example, a 'begin-paragraph' tag always has a corresponding 'end-paragraph' tag. If a tag had to be added programmatically (for example, if the requested passage ends in the middle of a paragraph), a virtual="virtual" attribute is added to the tag (for example, 'end-paragraph virtual="virtual"'). If you want to know whether a given tag was added programmatically and doesn't really exist in the text, set this option to true. (default: false)
        /// </summary>
        public Boolean IncludeVirtualAttributes { get; set; }

        //- @IncludeXmlDeclaration -//
        /// <summary>
        /// Include an XML declaration at the top of the document. (default: false)
        /// </summary>
        public Boolean IncludeXmlDeclaration { get; set; }

        //+
        //- $BaseElementDefault -//
        private String BaseElementDefault { get; set; }

        //- $IncludeXmlDeclarationDefault -//
        private Boolean IncludeXmlDeclarationDefault { get; set; }

        //- $IncludeDocTypeDefault -//
        private Boolean IncludeDocTypeDefault { get; set; }

        //- $IncludeQuoteEntitiesDefault -//
        private Boolean IncludeQuoteEntitiesDefault { get; set; }

        //- $IncludeSimpleEntitiesDefault -//
        private Boolean IncludeSimpleEntitiesDefault { get; set; }

        //- $IncludeCrossReferencesDefault -//
        private Boolean IncludeCrossReferencesDefault { get; set; }

        //- $IncludeLineBreaksDefault -//
        private Boolean IncludeLineBreaksDefault { get; set; }

        //- $IncludeWordIdsDefault -//
        private Boolean IncludeWordIdsDefault { get; set; }

        //- $IncludeVirtualAttributesDefault -//
        private Boolean IncludeVirtualAttributesDefault { get; set; }

        //+
        //- @Timeout -//
        /// <summary>
        /// Time in seconds to wait before failing the service call
        /// </summary>
        public Int32 Timeout { get; set; }

        //+
        //- @Ctor -//
        public CrosswayXmlVersion10Setting()
        {
            this.IncludeXmlDeclaration = false;
            this.IncludeDocType = true;
            this.IncludeQuoteEntities = true;
            this.IncludeSimpleEntities = true;
            this.IncludeCrossReferences = false;
            this.IncludeLineBreaks = true;
            this.IncludeWordIds = false;
            this.IncludeVirtualAttributes = false;

            this.IncludeXmlDeclarationDefault = false;
            this.IncludeDocTypeDefault = true;
            this.IncludeQuoteEntitiesDefault = true;
            this.IncludeSimpleEntitiesDefault = false;
            this.IncludeCrossReferencesDefault = false;
            this.IncludeLineBreaksDefault = true;
            this.IncludeWordIdsDefault = false;
            this.IncludeVirtualAttributesDefault = false;
        }

        //+
        //- $IDefaultOptimization.GetSettingsList -//
        Dictionary<String, String> IDefaultOptimization.GetSettingsList()
        {
            Dictionary<String, String> d = new Dictionary<String, String>();
            //+
            ArgumentListManager.AddArgument(d, "output-format", "crossway-xml-1.0");
            if (this.IncludeXmlDeclaration != this.IncludeXmlDeclarationDefault) ArgumentListManager.AddArgument(d, "include-xml-declaration", this.IncludeXmlDeclaration);
            if (this.IncludeDocType != this.IncludeDocTypeDefault) ArgumentListManager.AddArgument(d, "include-doctype", this.IncludeDocType);
            if (this.IncludeQuoteEntities != this.IncludeQuoteEntitiesDefault) ArgumentListManager.AddArgument(d, "include-quote-entities", this.IncludeQuoteEntities);
            if (this.IncludeSimpleEntities != this.IncludeSimpleEntitiesDefault) ArgumentListManager.AddArgument(d, "include-simple-entities", this.IncludeSimpleEntities);
            if (this.IncludeCrossReferences != this.IncludeCrossReferencesDefault) ArgumentListManager.AddArgument(d, "include-cross-references", this.IncludeCrossReferences);
            if (this.IncludeLineBreaks != this.IncludeLineBreaksDefault) ArgumentListManager.AddArgument(d, "include-line-breaks", this.IncludeLineBreaks);
            if (this.IncludeWordIds != this.IncludeWordIdsDefault) ArgumentListManager.AddArgument(d, "include-word-ids", this.IncludeWordIds);
            if (this.IncludeVirtualAttributes != this.IncludeVirtualAttributesDefault) ArgumentListManager.AddArgument(d, "include-virtual-attributes", this.IncludeVirtualAttributes);
            if (!String.IsNullOrEmpty(this.BaseElement)) ArgumentListManager.AddArgument(d, "base-element", this.BaseElement);
            //+
            return d;
        }
    }
}